# \file ext/FindOracle.cmake
#
# This module finds if Oracle is installed and determines where the
# include files and libraries for Pro*C are. This code sets the 
# following variables:
#
#  ORACLE_PROC_LIBRARY            = full path to the Pro*C library (orasql.lib)
#  ORACLE_PROC_INCLUDE_PATH       = path to where sqlca.h can be found
#  ORACLE_PROC_BIN                = full path to the precomp binary
#
# \date 05/13/2003
# \author Anilkumar Patro

# Find program for the Pro*C precompiler
FIND_PROGRAM(ORACLE_PROC_BIN NAMES precomp)

IF (WIN32)
  MARK_AS_ADVANCED(
    ORACLE_PROC_BIN
    )
ENDIF(WIN32)

# Find the Pro*C library path
GET_FILENAME_COMPONENT(ORACLE_PROC_BIN_PATH ${ORACLE_PROC_BIN} PATH)

SET (ORACLE_PROC_POSSIBLE_LIB_PATHS
  "${ORACLE_PROC_BIN_PATH}/../precomp/lib"
  "${ORACLE_PROC_BIN_PATH}/../precomp/lib/msvc"
  /usr/lib 
  /usr/local/lib
  /usr/lib/ora81
  /usr/local/lib
  "C:/Program Files/Oracle/ora81/precomp/lib" 
  "C:/Program Files/Oracle/ora81/precomp/lib/msvc" 
  "C:/Oracle/ora81/precomp/precomp/lib" 
  "C:/Oracle/ora81/precomp/precomp/lib/msvc" 
  [HKEY_LOCAL_MACHINE\\SOFTWARE\\Oracle;ORACLE_HOME]/precomp/lib
  [HKEY_LOCAL_MACHINE\\SOFTWARE\\Oracle;ORACLE_HOME]/precomp/lib/msvc
)

FIND_LIBRARY(ORACLE_PROC_LIBRARY
  NAMES orasql
  PATHS ${ORACLE_PROC_POSSIBLE_LIB_PATHS}
)

# Find the Pro*C include path
SET (ORACLE_PROC_POSSIBLE_INCLUDE_PATHS
  "${ORACLE_PROC_BIN_PATH}/../precomp/public"
  /usr/include 
  /usr/local/include
  "C:/Program Files/Oracle/ora81/precomp/public" 
  "C:/Oracle/ora81/precomp/public" 
  [HKEY_LOCAL_MACHINE\\SOFTWARE\\Oracle;ORACLE_HOME]/precomp/public
)

FIND_PATH(ORACLE_PROC_INCLUDE_PATH sqlca.h 
  ${ORACLE_PROC_POSSIBLE_INCLUDE_PATHS}
)

# Log:
# $Log: FindOracle.cmake,v $
# Revision 1.1  2003/09/01 23:11:25  anil
# Initial import of project tree.
#
#
