
#include <xmdv/okcdata.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <xmdv/macros.h>
#include <xmdv/brush.h>
#include <math.h>
#include <search.h>
#include <xmdv/XmdvTool.h>
#include <xmdv/glyph_place.h>
#include <xmdv/Analysis.h>


int IncreaseSort( const void *elm1, const void *elm2)
{
	double value1 = ((SortStruct *)elm1)->value;
	double value2 = ((SortStruct *)elm2)->value;

	if ( value1 < value2 )
		return -1;
	else if ( value1 == value2 )
		return 0;
	else
		return 1;
}


int DecreaseSort( const void *elm1, const void *elm2)
{
	double value1 = ((SortStruct *)elm1)->value;
	double value2 = ((SortStruct *)elm2)->value;

	if ( value1 < value2 )
		return 1;
	else if ( value1 == value2 )
		return 0;
	else
		return -1;
}


//decide the base of every glyph according to their order in one dimension
void okcdata::Placement_Sort_Glyph(int dimNumber, bool increase)
{
	SortStruct *sortArray = new SortStruct[data_size];
	
	int i, k;
	k = dimNumber;
	for ( i = 0; i < data_size; i++ )			
	{
		sortArray[i].number = i;
		sortArray[i].value = data_buf[k];
		k = k+all_dims;
	}

	if ( increase == true )
		qsort( sortArray, data_size, sizeof( SortStruct ), IncreaseSort );
	else 
		qsort( sortArray, data_size, sizeof( SortStruct ), DecreaseSort );

    blocksize = 1.0/(1.0 + sqrt(data_size));
    blockcnt = int( .5 + sqrt(data_size) );
    halfblocksize = .5 * blocksize;

	for ( i = 0; i < data_size; i++ )
	{
		basex[sortArray[i].number] = blocksize * (i%blockcnt) + halfblocksize;
		basey[sortArray[i].number] = blocksize * (i/blockcnt) + halfblocksize;
	}

	delete sortArray;
	
}


//decide the base of every glyph according to their order in the file.
void okcdata::Placement_Original_Glyph()
{
    blocksize = 1.0/(1.0 + sqrt(data_size));
    blockcnt = int( .5 + sqrt(data_size) );
    halfblocksize = .5 * blocksize;

	for ( int i = 0; i < data_size; i++ )
	{
		basex[i] = blocksize * (i%blockcnt) + halfblocksize;
		basey[i] = blocksize * (i/blockcnt) + halfblocksize;
	}
	
}


void okcdata::AddBuffer()
{
    double radio = 1 - blocksize;
	for ( int i = 0; i < data_size; i++ )
	{
		basex[i] = halfblocksize + basex[i]*radio;
		basey[i] = halfblocksize + basey[i]*radio;
	}
}



//decide the base of every glyph according to their order in the file.
void okcdata::Placement_DataDriven_Glyph(int XdimNumber, int YdimNumber)
{
	double xmin = all_dim_min[XdimNumber]; 
	double ymin = all_dim_min[YdimNumber]; 
	double xpara = 1.0/(all_dim_max[XdimNumber] - xmin);
	double ypara = 1.0/(all_dim_max[YdimNumber] - ymin);
	
	int base = 0, dim = all_dims;
	for ( int i = 0; i < data_size; i++ )
	{
		basex[i] = (data_buf[base+XdimNumber] - xmin) * xpara;
		basey[i] = 1-(data_buf[base+YdimNumber] - ymin) * ypara;
		base += all_dims;
	}

    AddBuffer();
	
}

/*************************************************************************/
//Code for glyph shift

/* program to adjust the positions of glyphs to reduce or eliminate overlaps.
Overlapping points exert a force at each other in the direction of the
vector between them, weighted by how close they are to each other.  Multiple
overlaps are simply summed and averaged.  Written by Matt Ward, last changed 8/2/00
*/

#define MAX_ITER 10             /* maximum number of iterations of adjustment(change as needed) */
#define SIZE 25                 /* maximum size of a glyph in x or y (change as needed) */
#define TOLERANCE 2             /* maximum allowable percentage of glyph overlaps (can be changed) */
#define SPEED SIZE/4            /* maximum change in position per iteration (arbitrary)*/

/* macro to determine if both x and y extents overlap between 2 glyphs - may have it backwards */       
#define OVER(a, b)  (((abs(pos[a].x-pos[b].x) < SIZE) && (abs(pos[a].y-pos[b].y) < SIZE))? 1:0)

struct position {               /* structure to hold a position */
   int x, y;
   };

struct position *pos;           /* buffer to hold glyph positions */
struct position *tpos;          /* temporary buffer for positions */

int num_pts;                    /* number of data points */

/* initialize space for positions and store them */
void init(int pts, double *basex, double *basey, double para)
{
	num_pts = pts;
	pos = (struct position *)malloc(pts * sizeof(struct position));
	tpos = (struct position *)malloc(pts * sizeof(struct position));
	if(pos == NULL || tpos == NULL)
	{
	  printf("Malloc failed in init\n");
	  exit(-1);
	}
/* now fill in positions of glyphs, whether data driven or derived */
/* CODE TO INITIALIZE POSITIONS GOES HERE */
	for ( int i = 0; i < pts; i++ )
	{
		pos[i].x = tpos[i].x = int ( 0.5 + basex[i]*para);
		pos[i].y = tpos[i].y = int ( 0.5 + basey[i]*para);
	}
}

/* count number of overlaps based on overlapping extents */
int num_overlaps(void)
{
   int i, j, num = 0;
   for(i = 0;i < num_pts;i++)
      for(j = i+1;j < num_pts;j++)
         num += OVER(i, j);
   return(num);
}

/* need to determine which points to move, what direction, and how much.
Points to move are those that overlap.  The direction of motion for point i is the
combined forces from all points that overlap it, formed as the vectors going
from the overlapping point to point i.  The vectors are weighted, so closer
points have more influence on the direction.  The overall motion is weighted by
SPEED, the maximum number of pixels a glyph can move */
void move_glyphs(void)
{
	int i, j, num;
	double mag;
	double vector[2], sumvector[2], maxsize;
	maxsize = (double)SIZE * 1.415;       /* max distance between overlap centers */
	sumvector[0] = sumvector[1] = 0.0;
	for(i = 0;i < num_pts;i++)   
	{
		num = 0;
/* for every point... */
		for(j = 0;j < num_pts;j++)        
		{
/* if it has any overlap */
			if(i != j && OVER(i, j))       
			{
				num++;
/* calculate vector between centers (the direction of force repelling points) */

/* if points are at same location, add a random displacement */
				if((pos[j].x == pos[i].x) && (pos[j].y == pos[i].y)) 
				{
					vector[0] = -1.0 + 2.0 * rand();        /* rand num between -1 and 1 */
					vector[1] = -1.0 + 2.0 * rand();
					mag = sqrt((vector[0] * vector[0]) + (vector[1] * vector[1]));
					/* normalize, and add with full weight to sumvector */
					vector[0] = vector[0]/mag;
					vector[1] = vector[1]/mag;
					sumvector[0] += vector[0];
					sumvector[1] += vector[1];
				}
/* normalize vector and add weighted amount to sumvector */
				else 
				{
					vector[0] = pos[i].x - pos[j].x;
					vector[1] = pos[i].y - pos[j].y;
					mag = sqrt((vector[0] * vector[0]) + (vector[1] * vector[1]));
					vector[0] = vector[0]/mag;
					vector[1] = vector[1]/mag;
					/* weight decreases as distance increases, from 1.0 to 0.0 */
					sumvector[0] += (1.0-mag/maxsize) * vector[0];
					sumvector[1] += (1.0-mag/maxsize) * vector[1];
				}
			}
		}
/* NOTE: may want to add a minor random offset for points being moved, to avoid
   oscillating between 2 locations */

/* average the vectors (this is not exactly correct, as lots of many nearby
points can dillute influence of very close one, resulting in smaller move, but in
correct direction)  */
		if(num > 0)       
		{
			sumvector[0] = sumvector[0]/(double)num;
			sumvector[1] = sumvector[1]/(double)num;
		}
/* calculate the new position for glyph i */
		tpos[i].x = (int)(pos[i].x + SPEED * sumvector[0]);
		tpos[i].y = (int)(pos[i].y + SPEED * sumvector[1]);
	}
/* now copy everything back into pos array */
	for(i = 0;i < num_pts;i++)   
	{
		pos[i].x = tpos[i].x;
		pos[i].y = tpos[i].y;
	}
}


/* compute tolerance as number of allowable overlaps and iteratively move
     points until tolerance met or maximum iterations reached */
void adjust(void)
{
   int i, tol;
   tol = TOLERANCE * num_pts / 100; 
   for(i = 0;i < MAX_ITER && num_overlaps() > tol;i++)
      move_glyphs();
}

void finished(int pts, double *basex, double *basey, double para)
{
//put back the glyph basex and basey
	for ( int i = 0; i < pts; i++ )
	{
		basex[i] = pos[i].x/para;
		basey[i] = pos[i].y/para;
	}
}


//decide the base of every glyph after reducing overlap.
void okcdata::Placement_Shift_Glyph()
{
	double para = SIZE/blocksize;
	init(data_size, basex, basey, para);	
	adjust();
	finished(data_size, basex, basey, para);
}


//////////////////////////////////////////////////////////////////////////////////
// code for space the glyphs

#define MAX_SPACE 4 /* maximum number of blank glyphs to put between glyphs */

static int *spaces, total_space;       /* spaces between adjacent points, and total space */
static double *dist;                    /* distances between adjacent points */

/* calculate the distance between 2 glyphs as the square root of the sum of the normalized
   squared difference in all the dimensions */

double comp_dist(ARR<double> dataA, ARR<double> dataB, ARR<double> maxmin, int numdims)
{
	double result = 0;
	int i;
	for(i = 0;i < numdims;i++)
	{
		if ( fabs(maxmin[i]) > 1e-9 )
			result += ((dataA[i] - dataB[i]) * (dataA[i] - dataB[i])) / maxmin[i];
	}
	return(sqrt(result));
}

void okcdata::Placement_Original_Space_Glyph()
{
  int i, index;
  double maxdist = 0.0;
  int row_length;       /* number of glyphs that will fit on a row */

/* get memory for lists */
  int numpts = data_size;

  spaces = new int[numpts];
  dist = new double[numpts];

/* find the distance between adjacent points, find maximum */
	ARR<double> dataA(dims);
	ARR<double> dataB(dims);
	ARR<double> maxmin(dims);

	for ( i = 0; i < dims; i++ )
	{
		maxmin[i] = (dim_max[i] - dim_min[i]) * (dim_max[i] - dim_min[i]);
	}

	for(i = 0;i < numpts-1;i++)   
	{
		GetData(dataA,i);
		GetData(dataB,i+1);
		dist[i] = comp_dist(dataA, dataB, maxmin, dims);
		if(dist[i] > maxdist)
			maxdist = dist[i];
	}

/* put spacing proportional to their distances, and calculate total space needs
*/
  total_space = 0;
  for(i = 0;i < numpts-1;i++)   {
    spaces[i] = (int) (1 + MAX_SPACE * dist[i] / maxdist);
    total_space += spaces[i];
    }

  row_length = (int) (sqrt((double)total_space) + .5);  /* number of glyphs that will fit in
        a row, assuming a square canvas */

  blocksize = 1.0 / row_length;     /* max size of a glyph, assumes square canvas */
  halfblocksize = blocksize/2.0;            /* pos is where center of glyph is, so need offset */

/* comput the locations of the glyphs based on how much space is between them
*/
  index = 0;
  for(i = 0;i < numpts;i++)     {
    basex[i] = (index % row_length) * blocksize + halfblocksize;
    basey[i] = (index / row_length) * blocksize + halfblocksize;
    index += spaces[i];
    }
 /* now you can draw the glyphs at locations stored in pos */
}


//decide the base of every glyph according to their order in one dimension
void okcdata::Placement_Sort_Space_Glyph(int dimNumber, bool increase)
{
	SortStruct *sortArray = new SortStruct[data_size];
	
	int i, k;
	k = dimNumber;
	for ( i = 0; i < data_size; i++ )			
	{
		sortArray[i].number = i;
		sortArray[i].value = data_buf[k];
		k = k+all_dims;
	}

	if ( increase == true )
		qsort( sortArray, data_size, sizeof( SortStruct ), IncreaseSort );
	else 
		qsort( sortArray, data_size, sizeof( SortStruct ), DecreaseSort );

	int index;
	double maxdist = 0.0;
	int row_length;       /* number of glyphs that will fit on a row */

	/* get memory for lists */
	int numpts = data_size;

	spaces = new int[numpts];
	dist = new double[numpts];

	/* find the distance between adjacent points, find maximum */
	ARR<double> dataA(dims);
	ARR<double> dataB(dims);
	ARR<double> maxmin(dims);

	for ( i = 0; i < dims; i++ )
	{
		maxmin[i] = (dim_max[i] - dim_min[i]) * (dim_max[i] - dim_min[i]);
	}

	for(i = 0;i < numpts-1;i++)   
	{
		GetData(dataA,sortArray[i].number);
		GetData(dataB,sortArray[i+1].number);
		dist[i] = comp_dist(dataA, dataB, maxmin, dims);
		if(dist[i] > maxdist)
			maxdist = dist[i];
	}

	/* put spacing proportional to their distances, and calculate total space needs 	*/
	total_space = 0;
	for(i = 0;i < numpts-1;i++)   
	{
		spaces[i] = (int) (1 + MAX_SPACE * dist[i] / maxdist);
		total_space += spaces[i];
	}

	row_length = (int) (sqrt((double)total_space) + .5);  /* number of glyphs that will fit in
	a row, assuming a square canvas */

	blocksize = 1.0 / row_length;     /* max size of a glyph, assumes square canvas */
	halfblocksize = blocksize/2.0;            /* pos is where center of glyph is, so need offset */

	/* comput the locations of the glyphs based on how much space is between them	*/
	index = 0;
	for(i = 0;i < numpts;i++)     
	{
		basex[sortArray[i].number] = (index % row_length) * blocksize + halfblocksize;
		basey[sortArray[i].number] = (index / row_length) * blocksize + halfblocksize;
		index += spaces[i];
	}
 /* now you can draw the glyphs at locations stored in pos */

	delete sortArray;
	
}



void ReadDataFromBuffer( double **data, int n, int m )
{
	for ( int i = 0; i < n; i++ )
	{
	    ARR<double> datapoint(m);
        dataset->GetData(datapoint, i);
		for ( int j = 0; j < m; j++ )
		{
			data[i][j] = datapoint[j];
		}
	}
}

bool principal_Components_Analysis_dims(int prin_num, double **prin, double *prin_max, double *prin_min)
{
	int n, m;
	double **data;
	
	n = dataset->data_size;
	m = dataset->dims;

	data = matrix(n, m);
	ReadDataFromBuffer(data, n, m);

    bool success = principal_Components_Analysis(n, m, data, prin_num, prin, prin_max, prin_min);

	free_matrix(data, n, m); 
	return success;
}


void okcdata::Placement_Derived_Glyph(int XdimNumber, int YdimNumber)
{
	int prin_num;
	double **principal;
	double *principal_max;
	double *principal_min;
	int i;

	if ( dataset->dims < 3 )
		prin_num = dims;
	else
		prin_num = 3;

	principal = new double *[prin_num];
	for ( i = 0; i < prin_num; i++ )
		principal[i] = new double[data_size];
	principal_max = new double[prin_num];
	principal_min = new double[prin_num];

	if ( !principal_Components_Analysis_dims(prin_num, principal, principal_max, principal_min) )
	{
		for ( i = 0; i < prin_num; i++ )
			delete principal[i];
		delete principal;
		delete principal_max;
		delete principal_min;
		return;
	}

	int base;

	if ( XdimNumber < all_dims )
	{
		double xmin = all_dim_min[XdimNumber]; 
		double xpara = 1.0/(all_dim_max[XdimNumber] - xmin);
		
		base = 0;
		for ( i = 0; i < data_size; i++ )
		{
			basex[i] = (data_buf[base+XdimNumber] - xmin) * xpara;
			base += all_dims;
		}
	}
	else
	{
		XdimNumber -= all_dims;
		double xmin = principal_min[XdimNumber]; 
		double xpara = 1.0/(principal_max[XdimNumber] - xmin);
		
		for ( i = 0; i < data_size; i++ )
			basex[i] = (principal[XdimNumber][i] - xmin) * xpara;
	}
	
	if ( YdimNumber < all_dims )
	{
		double ymin = all_dim_min[YdimNumber]; 
		double ypara = 1.0/(all_dim_max[YdimNumber] - ymin);
		
		base = 0;
		for ( i = 0; i < data_size; i++ )
		{
			basey[i] = 1-(data_buf[base+YdimNumber] - ymin) * ypara;
			base += all_dims;
		}
	}
	else
	{
		YdimNumber -= all_dims;
		double ymin = principal_min[YdimNumber]; 
		double ypara = 1.0/(principal_max[YdimNumber] - ymin);
		
		for ( i = 0; i < data_size; i++ )
			basey[i] = 1-(principal[YdimNumber][i] - ymin) * ypara;
	}
    
    for ( i = 0; i < prin_num; i++ )
		delete principal[i];
	delete principal;
	delete principal_max;
	delete principal_min;

    AddBuffer();
	
}


void okcdata::GetPrin0(double* &prin0)
{
	int prin_num;
	double **principal;
	double *principal_max;
	double *principal_min;
	int i;

	if ( dataset->dims < 3 )
		prin_num = dims;
	else
		prin_num = 3;

	principal = new double *[prin_num];
	for ( i = 0; i < prin_num; i++ )
		principal[i] = new double[data_size];
	principal_max = new double[prin_num];
	principal_min = new double[prin_num];

	if ( !principal_Components_Analysis_dims(prin_num, principal, principal_max, principal_min) )
	{
		for ( i = 0; i < prin_num; i++ )
			delete principal[i];
		delete principal;
		delete principal_max;
		delete principal_min;
		return;
	}

    prin0 = new double[data_size];
    for ( i = 0; i < data_size; i++ )
		prin0[i] = principal[0][i];

    for ( i = 0; i < prin_num; i++ )
		delete principal[i];
	delete principal;
	delete principal_max;
	delete principal_min;
	
}
